defmodule SqlMembershipProvider.Role do
  @moduledoc """
  Struct for representing a security role that a user may assume
  """

  import SqlMembershipProvider.TdsUtils

  defstruct [
    :application_id,
    :role_id,
    :role_name,
    :lowered_role_name,
    :description
  ]

  @type t() :: %SqlMembershipProvider.Role{
          application_id: String.t(),
          role_id: String.t(),
          role_name: String.t(),
          lowered_role_name: String.t(),
          description: String.t() | nil
        }

  @doc """
  Fetch all roles that a user belongs to.
  """
  @spec find_by_user_id(atom() | pid(), String.t()) :: [SqlMembershipProvider.Role.t()]
  def find_by_user_id(pid, user_id) when (is_atom(pid) or is_pid(pid)) and is_binary(user_id) do
    Tds.query!(
      pid,
      "SELECT
        r.ApplicationId as application_id,
        r.RoleId as role_id,
        r.RoleName as role_name,
        r.LoweredRoleName as lowered_role_name,
        r.Description as description
        FROM aspnet_UsersInRoles AS uir
        JOIN aspnet_Roles AS r ON r.RoleId = uir.RoleId
        WHERE uir.UserId = @user_id",
      [%Tds.Parameter{name: "@user_id", value: user_id}]
    )
    |> result_to_map()
    |> stringify_uuids(["application_id", "role_id"])
    |> to_struct(__MODULE__)
  end
end
